@@ -17,8 +17,13 @@
 #define QGSGEOMETRYGENERATORSYMBOLLAYER_H
 
 #include "qgis_core.h"
+#include "qgis.h"
 #include "qgssymbollayer.h"
 
+class QgsFillSymbol;
+class QgsLineSymbol;
+class QgsMarkerSymbol;
+
 /**
  * \ingroup core
  * \class QgsGeometryGeneratorSymbolLayer
@@ -28,7 +33,8 @@ class CORE_EXPORT QgsGeometryGeneratorSymbolLayer : public QgsSymbolLayer
   public:
     ~QgsGeometryGeneratorSymbolLayer() override;
 
-    static QgsSymbolLayer *create( const QgsStringMap &properties ) SIP_FACTORY;
+    //! Creates the symbol layer
+    static QgsSymbolLayer *create( const QVariantMap &properties ) SIP_FACTORY;
 
     QString layerType() const override;
 
@@ -38,7 +44,7 @@ class CORE_EXPORT QgsGeometryGeneratorSymbolLayer : public QgsSymbolLayer
      *
      * \param symbolType The symbol type which shall be used below this symbol.
      */
-    void setSymbolType( QgsSymbol::SymbolType symbolType );
+    void setSymbolType( Qgis::SymbolType symbolType );
 
     /**
      * Access the symbol type. This defines the type of geometry
@@ -46,17 +52,21 @@ class CORE_EXPORT QgsGeometryGeneratorSymbolLayer : public QgsSymbolLayer
      *
      * \returns Symbol type
      */
-    QgsSymbol::SymbolType symbolType() const { return mSymbolType; }
+    Qgis::SymbolType symbolType() const { return mSymbolType; }
 
     void startRender( QgsSymbolRenderContext &context ) override;
     void stopRender( QgsSymbolRenderContext &context ) override;
     void startFeatureRender( const QgsFeature &feature, QgsRenderContext &context ) override;
     void stopFeatureRender( const QgsFeature &feature, QgsRenderContext &context ) override;
     bool usesMapUnits() const override;
+    QColor color() const override;
+    QgsUnitTypes::RenderUnit outputUnit() const override;
+    void setOutputUnit( QgsUnitTypes::RenderUnit unit ) override;
+    QgsMapUnitScale mapUnitScale() const override;
 
     QgsSymbolLayer *clone() const override SIP_FACTORY;
 
-    QgsStringMap properties() const override;
+    QVariantMap properties() const override;
 
     void drawPreviewIcon( QgsSymbolRenderContext &context, QSize size ) override;
 
@@ -68,7 +78,29 @@ class CORE_EXPORT QgsGeometryGeneratorSymbolLayer : public QgsSymbolLayer
     /**
      * Gets the expression to generate this geometry.
      */
-    QString geometryExpression() const { return mExpression->expression(); }
+    QString geometryExpression() const;
+
+    /**
+     * Returns the unit for the geometry expression.
+     *
+     * By default this is QgsUnitTypes::MapUnits, which means that the geometryExpression()
+     * will return geometries in the associated layer's CRS.
+     *
+     * \see setUnits()
+     * \since QGIS 3.22
+     */
+    QgsUnitTypes::RenderUnit units() const { return mUnits; }
+
+    /**
+     * Sets the \a units for the geometry expression.
+     *
+     * By default this is QgsUnitTypes::MapUnits, which means that the geometryExpression()
+     * will return geometries in the associated layer's CRS.
+     *
+     * \see units()
+     * \since QGIS 3.22
+     */
+    void setUnits( QgsUnitTypes::RenderUnit units ) { mUnits = units;}
 
     QgsSymbol *subSymbol() override { return mSymbol; }
 
@@ -91,10 +123,12 @@ class CORE_EXPORT QgsGeometryGeneratorSymbolLayer : public QgsSymbolLayer
      * which contains a QgsRenderContext which in turn contains an expression
      * context which is available to the evaluated expression.
      *
-     * \param context The rendering context which will be used to render and to
-     *                construct a geometry.
+     * \param context The rendering context which will be used to render and to construct a geometry.
+     * \param geometryType type of original geometry being rendered by the parent symbol (since QGIS 3.22)
+     * \param points optional list of original points which are being rendered by the parent symbol (since QGIS 3.22)
+     * \param rings optional list of original rings which are being rendered by the parent symbol (since QGIS 3.22)
      */
-    virtual void render( QgsSymbolRenderContext &context );
+    void render( QgsSymbolRenderContext &context, QgsWkbTypes::GeometryType geometryType = QgsWkbTypes::GeometryType::UnknownGeometry, const QPolygonF *points = nullptr, const QVector<QPolygonF> *rings = nullptr );
 
     void setColor( const QColor &color ) override;
 
@@ -105,6 +139,17 @@ class CORE_EXPORT QgsGeometryGeneratorSymbolLayer : public QgsSymbolLayer
     QgsGeometryGeneratorSymbolLayer( const QgsGeometryGeneratorSymbolLayer &copy );
 #endif
 
+    /**
+     * Input geometry must be in painter units!
+     */
+    QgsGeometry evaluateGeometryInPainterUnits( const QgsGeometry &input, const QgsFeature &feature, const QgsRenderContext &renderContext, QgsExpressionContext &expressionContext ) const;
+
+    /**
+     * Tries to coerce the geometry output by the generator expression into
+     * a type usable by the symbol.
+     */
+    QgsGeometry coerceToExpectedType( const QgsGeometry &geometry ) const;
+
     std::unique_ptr<QgsExpression> mExpression;
     std::unique_ptr<QgsFillSymbol> mFillSymbol;
     std::unique_ptr<QgsLineSymbol> mLineSymbol;
@@ -114,7 +159,9 @@ class CORE_EXPORT QgsGeometryGeneratorSymbolLayer : public QgsSymbolLayer
     /**
      * The type of the sub symbol.
      */
-    QgsSymbol::SymbolType mSymbolType;
+    Qgis::SymbolType mSymbolType;
+
+    QgsUnitTypes::RenderUnit mUnits = QgsUnitTypes::RenderMapUnits;
 
     bool mRenderingFeature = false;
     bool mHasRenderedFeature = false;
