@@ -28,7 +28,7 @@ class QgsVectorLayer;
 
 /**
  * \ingroup core
- * A layout table subclass that displays attributes from a vector layer.
+ * \brief A layout table subclass that displays attributes from a vector layer.
  * \since QGIS 3.0
 */
 class CORE_EXPORT QgsLayoutItemAttributeTable: public QgsLayoutTable
@@ -127,7 +127,7 @@ class CORE_EXPORT QgsLayoutItemAttributeTable: public QgsLayoutTable
     /**
      * Sets a layout \a map to use to limit the extent of features shown in the
      * attribute table. This setting only has an effect if setDisplayOnlyVisibleFeatures is
-     * set to true. Changing the map forces the table to refetch features from its
+     * set to TRUE. Changing the map forces the table to refetch features from its
      * vector layer, and may result in the table changing size to accommodate the new displayed
      * feature attributes.
      * \see map()
@@ -137,7 +137,7 @@ class CORE_EXPORT QgsLayoutItemAttributeTable: public QgsLayoutTable
 
     /**
      * Returns the layout map whose extents are controlling the features shown in the
-     * table. The extents of the map are only used if displayOnlyVisibleFeatures() is true.
+     * table. The extents of the map are only used if displayOnlyVisibleFeatures() is TRUE.
      * \see setMap()
      * \see displayOnlyVisibleFeatures()
      */
@@ -160,7 +160,7 @@ class CORE_EXPORT QgsLayoutItemAttributeTable: public QgsLayoutTable
     /**
      * Sets attribute table to only show unique rows.
      *
-     * Set \a uniqueOnly to true to show only unique rows. Duplicate rows
+     * Set \a uniqueOnly to TRUE to show only unique rows. Duplicate rows
      * will be stripped from the table.
      *
      * \see uniqueRowsOnly()
@@ -168,7 +168,7 @@ class CORE_EXPORT QgsLayoutItemAttributeTable: public QgsLayoutTable
     void setUniqueRowsOnly( bool uniqueOnly );
 
     /**
-     * Returns true if the table is set to show only unique rows.
+     * Returns TRUE if the table is set to show only unique rows.
      *
      * \see setUniqueRowsOnly()
      */
@@ -185,7 +185,7 @@ class CORE_EXPORT QgsLayoutItemAttributeTable: public QgsLayoutTable
     void setDisplayOnlyVisibleFeatures( bool visibleOnly );
 
     /**
-     * Returns true if the table is set to show only features visible on a corresponding
+     * Returns TRUE if the table is set to show only features visible on a corresponding
      * map item.
      *
      * \see map()
@@ -202,7 +202,7 @@ class CORE_EXPORT QgsLayoutItemAttributeTable: public QgsLayoutTable
     void setFilterToAtlasFeature( bool filterToAtlas );
 
     /**
-     * Returns true if the table is set to only show features which intersect the current atlas
+     * Returns TRUE if the table is set to only show features which intersect the current atlas
      * feature.
      *
      * \see setFilterToAtlasFeature()
@@ -210,7 +210,7 @@ class CORE_EXPORT QgsLayoutItemAttributeTable: public QgsLayoutTable
     bool filterToAtlasFeature() const { return mFilterToAtlasIntersection; }
 
     /**
-     * Returns true if a feature filter is active on the attribute table.
+     * Returns TRUE if a feature filter is active on the attribute table.
      *
      * \see setFilterFeatures()
      * \see featureFilter()
@@ -228,7 +228,7 @@ class CORE_EXPORT QgsLayoutItemAttributeTable: public QgsLayoutTable
 
     /**
      * Returns the current expression used to filter features for the table. The filter is only
-     * active if filterFeatures() is true.
+     * active if filterFeatures() is TRUE.
      *
      * \see setFeatureFilter()
      * \see filterFeatures()
@@ -237,7 +237,7 @@ class CORE_EXPORT QgsLayoutItemAttributeTable: public QgsLayoutTable
 
     /**
      * Sets the \a expression used for filtering features in the table. The filter is only
-     * active if filterFeatures() is set to true. Changing this setting forces the table
+     * active if filterFeatures() is set to TRUE. Changing this setting forces the table
      * to refetch features from its vector layer, and may result in
      * the table changing size to accommodate the new displayed feature attributes.
      *
@@ -250,21 +250,12 @@ class CORE_EXPORT QgsLayoutItemAttributeTable: public QgsLayoutTable
      * Sets the attributes to display in the table.
      * \param fields list of fields names from the vector layer to show.
      * Set to an empty list to show all feature attributes.
-     * \param refresh set to true to force the table to refetch features from its vector layer
+     * \param refresh set to TRUE to force the table to refetch features from its vector layer
      * and immediately update the display of the table. This may result in the table changing size
      * to accommodate the new displayed feature attributes.
      */
     void setDisplayedFields( const QStringList &fields, bool refresh = true );
 
-    /**
-     * Returns the attributes used to sort the table's features.
-     * \returns a QList of integer/bool pairs, where the integer refers to the attribute index and
-     * the bool to the sort order for the attribute. If true the attribute is sorted ascending,
-     * if false, the attribute is sorted in descending order.
-     * \note not available in Python bindings
-     */
-    QVector< QPair<int, bool> > sortAttributes() const SIP_SKIP;
-
     /**
      * Sets a string to wrap the contents of the table cells by. Occurrences of this string will
      * be replaced by a line break.
@@ -283,27 +274,52 @@ class CORE_EXPORT QgsLayoutItemAttributeTable: public QgsLayoutTable
     /**
      * Queries the attribute table's vector layer for attributes to show in the table.
      * \param contents table content
-     * \returns true if attributes were successfully fetched
-     * \note not available in Python bindings
+     * \returns TRUE if attributes were successfully fetched
      */
-    bool getTableContents( QgsLayoutTableContents &contents ) override SIP_SKIP;
+    bool getTableContents( QgsLayoutTableContents &contents ) override;
+
+    QgsConditionalStyle conditionalCellStyle( int row, int column ) const override;
+    QgsTextFormat textFormatForCell( int row, int column ) const override;
+    QgsExpressionContextScope *scopeForCell( int row, int column ) const override SIP_FACTORY;
 
     QgsExpressionContext createExpressionContext() const override;
     void finalizeRestoreFromXml() override;
 
     void refreshDataDefinedProperty( QgsLayoutObject::DataDefinedProperty property = QgsLayoutObject::AllProperties ) override;
 
+    /**
+     * Returns TRUE if the attribute table will be rendered using the conditional styling
+     * properties of the linked vector layer.
+     *
+     * \see setUseConditionalStyling()
+     * \since QGIS 3.12
+     */
+    bool useConditionalStyling() const;
+
+    /**
+     * Sets whether the attribute table will be rendered using the conditional styling
+     * properties of the linked vector layer.
+     *
+     * \see useConditionalStyling()
+     * \since QGIS 3.12
+     */
+    void setUseConditionalStyling( bool enabled );
+
   protected:
 
     bool writePropertiesToElement( QDomElement &elem, QDomDocument &doc, const QgsReadWriteContext &context ) const override;
     bool readPropertiesFromElement( const QDomElement &itemElem, const QDomDocument &doc, const QgsReadWriteContext &context ) override;
 
+  private slots:
+
+    void disconnectCurrentMap();
+
   private:
 
     //! Attribute source
     ContentSource mSource = LayerAttributes;
     //! Associated vector layer
-    QgsVectorLayerRef mVectorLayer;
+    QgsVectorLayerRef mVectorLayer = nullptr;
 
     //! Data defined vector layer - only
     QPointer< QgsVectorLayer > mDataDefinedVectorLayer;
@@ -324,7 +340,7 @@ class CORE_EXPORT QgsLayoutItemAttributeTable: public QgsLayoutTable
     //! True if only unique rows should be shown
     bool mShowUniqueRowsOnly = false;
 
-    //! Shows only the features that are visible in the associated layout map (true by default)
+    //! Shows only the features that are visible in the associated layout map (TRUE by default)
     bool mShowOnlyVisibleFeatures = false;
 
     //! Shows only the features that intersect the current atlas feature
@@ -338,6 +354,25 @@ class CORE_EXPORT QgsLayoutItemAttributeTable: public QgsLayoutTable
 
     QString mWrapString;
 
+    bool mUseConditionalStyling = false;
+
+    QList< QList< QgsConditionalStyle > > mConditionalStyles;
+    QList< QgsFeature > mFeatures;
+    QMap<QString, QVariant> mLayerCache;
+
+    struct Cell
+    {
+      Cell() = default;
+
+      Cell( const QVariant &content, const QgsConditionalStyle &style, const QgsFeature &feature )
+        : content( content )
+        , style( style )
+        , feature( feature ) {}
+      QVariant content;
+      QgsConditionalStyle style;
+      QgsFeature feature;
+    };
+
     /**
      * Returns a list of attribute indices corresponding to displayed fields in the table.
      * \note kept for compatibility with 2.0 api only
@@ -357,6 +392,14 @@ class CORE_EXPORT QgsLayoutItemAttributeTable: public QgsLayoutTable
      */
     QVariant replaceWrapChar( const QVariant &variant ) const;
 
+#ifdef HAVE_SERVER_PYTHON_PLUGINS
+
+    /**
+     * Returns the list of visible columns filtered by feature filter provider.
+     */
+    QgsLayoutTableColumns filteredColumns( );
+#endif
+
   private slots:
     //! Checks if this vector layer will be removed (and sets mVectorLayer to 0 if yes)
     void removeLayer( const QString &layerId );
