@@ -29,41 +29,29 @@
 
 import json
 import os.path
-import warnings
 from urllib.request import build_opener, install_opener, ProxyHandler
 
 from qgis.PyQt.QtCore import Qt
-from qgis.PyQt.QtWidgets import (QApplication, QDialog, QComboBox,
+from qgis.PyQt.QtWidgets import (QDialog, QComboBox,
                                  QDialogButtonBox, QMessageBox,
                                  QTreeWidgetItem, QWidget)
-from qgis.PyQt.QtGui import QColor, QCursor
+from qgis.PyQt.QtGui import QColor
 
 from qgis.core import (QgsApplication, QgsCoordinateReferenceSystem,
                        QgsCoordinateTransform, QgsGeometry, QgsPointXY,
-                       QgsProviderRegistry, QgsSettings, QgsProject)
+                       QgsProviderRegistry, QgsSettings, QgsProject,
+                       QgsRectangle)
 from qgis.gui import QgsRubberBand, QgsGui
 from qgis.utils import OverrideCursor
 
-with warnings.catch_warnings():
-    warnings.filterwarnings("ignore", category=ResourceWarning)
-    warnings.filterwarnings("ignore", category=ImportWarning)
-    from owslib.csw import CatalogueServiceWeb  # spellok
-
-from owslib.fes import BBox, PropertyIsLike
-from owslib.ows import ExceptionReport
-
-try:
-    from owslib.util import Authentication
-except ImportError:
-    pass
-
 from MetaSearch import link_types
 from MetaSearch.dialogs.manageconnectionsdialog import ManageConnectionsDialog
 from MetaSearch.dialogs.newconnectiondialog import NewConnectionDialog
 from MetaSearch.dialogs.recorddialog import RecordDialog
-from MetaSearch.dialogs.xmldialog import XMLDialog
+from MetaSearch.dialogs.apidialog import APIRequestResponseDialog
+from MetaSearch.search_backend import get_catalog_service
 from MetaSearch.util import (clean_ows_url, get_connections_from_file,
-                             get_ui_class, get_help_url, highlight_xml,
+                             get_ui_class, get_help_url, highlight_content,
                              normalize_text, open_url, render_template,
                              serialize_string, StaticContext)
 
@@ -86,6 +74,7 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         self.catalog_url = None
         self.catalog_username = None
         self.catalog_password = None
+        self.catalog_type = None
         self.context = StaticContext()
 
         self.leKeywords.setShowSearchIcon(True)
@@ -98,18 +87,19 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         self.rubber_band.setWidth(5)
 
         # form inputs
-        self.startfrom = 0
-        self.maxrecords = 10
-        self.timeout = 10
-        self.disable_ssl_verification = False
+        self.startfrom = 1
         self.constraints = []
+        self.maxrecords = int(self.settings.value('/MetaSearch/returnRecords', 10))
+        self.timeout = int(self.settings.value('/MetaSearch/timeout', 10))
+        self.disable_ssl_verification = self.settings.value(
+            '/MetaSearch/disableSSL', False, bool)
 
-        # Servers tab
+        # Services tab
         self.cmbConnectionsServices.activated.connect(self.save_connection)
         self.cmbConnectionsSearch.activated.connect(self.save_connection)
         self.btnServerInfo.clicked.connect(self.connection_info)
         self.btnAddDefault.clicked.connect(self.add_default_connections)
-        self.btnCapabilities.clicked.connect(self.show_xml)
+        self.btnRawAPIResponse.clicked.connect(self.show_api)
         self.tabWidget.currentChanged.connect(self.populate_connection_list)
 
         # server management buttons
@@ -144,23 +134,41 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         self.mActionAddAms.triggered.connect(self.add_to_ows)
         self.mActionAddAfs.triggered.connect(self.add_to_ows)
         self.mActionAddGisFile.triggered.connect(self.add_gis_file)
-        self.btnShowXml.clicked.connect(self.show_xml)
+        self.btnViewRawAPIResponse.clicked.connect(self.show_api)
 
         self.manageGui()
 
     def manageGui(self):
         """open window"""
+        def _on_timeout_change(value):
+            self.settings.setValue('/MetaSearch/timeout', value)
+            self.timeout = value
+
+        def _on_records_change(value):
+            self.settings.setValue('/MetaSearch/returnRecords', value)
+            self.maxrecords = value
+
+        def _on_ssl_state_change(state):
+            self.settings.setValue('/MetaSearch/disableSSL', bool(state))
+            self.disable_ssl_verification = bool(state)
 
         self.tabWidget.setCurrentIndex(0)
         self.populate_connection_list()
-        self.btnCapabilities.setEnabled(False)
-        self.spnRecords.setValue(
-            int(self.settings.value('/MetaSearch/returnRecords', 10)))
+        self.btnRawAPIResponse.setEnabled(False)
+
+        # load settings
+        self.spnRecords.setValue(self.maxrecords)
+        self.spnRecords.valueChanged.connect(_on_records_change)
+        self.spnTimeout.setValue(self.timeout)
+        self.spnTimeout.valueChanged.connect(_on_timeout_change)
+        self.disableSSLVerification.setChecked(self.disable_ssl_verification)
+        self.disableSSLVerification.stateChanged.connect(_on_ssl_state_change)
 
         key = '/MetaSearch/%s' % self.cmbConnectionsSearch.currentText()
         self.catalog_url = self.settings.value('%s/url' % key)
         self.catalog_username = self.settings.value('%s/username' % key)
         self.catalog_password = self.settings.value('%s/password' % key)
+        self.catalog_type = self.settings.value('%s/catalog-type' % key)
 
         self.set_bbox_global()
 
@@ -169,7 +177,7 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         # install proxy handler if specified in QGIS settings
         self.install_proxy()
 
-    # Servers tab
+    # Services tab
 
     def populate_connection_list(self):
         """populate select box with connections"""
@@ -256,11 +264,12 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
             self.catalog_url = self.settings.value('%s/url' % key)
             self.catalog_username = self.settings.value('%s/username' % key)
             self.catalog_password = self.settings.value('%s/password' % key)
+            self.catalog_type = self.settings.value('%s/catalog-type' % key)
 
         if caller == 'cmbConnectionsServices':  # clear server metadata
             self.textMetadata.clear()
 
-        self.btnCapabilities.setEnabled(False)
+        self.btnRawAPIResponse.setEnabled(False)
 
     def connection_info(self):
         """show connection info"""
@@ -270,22 +279,23 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         self.catalog_url = self.settings.value('%s/url' % key)
         self.catalog_username = self.settings.value('%s/username' % key)
         self.catalog_password = self.settings.value('%s/password' % key)
+        self.catalog_type = self.settings.value('%s/catalog-type' % key)
 
         # connect to the server
-        if not self._get_csw():
+        if not self._get_catalog():
             return
 
         if self.catalog:  # display service metadata
-            self.btnCapabilities.setEnabled(True)
+            self.btnRawAPIResponse.setEnabled(True)
             metadata = render_template('en', self.context,
-                                       self.catalog,
-                                       'service_metadata.html')
+                                       self.catalog.conn,
+                                       self.catalog.service_info_template)
             style = QgsApplication.reportStyleSheet()
             self.textMetadata.clear()
             self.textMetadata.document().setDefaultStyleSheet(style)
             self.textMetadata.setHtml(metadata)
 
-            # clear results in Search tab
+            # clear results and disable buttons in Search tab
             self.clear_results()
 
     def add_connection(self):
@@ -308,8 +318,13 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         conn_edit.setWindowTitle(self.tr('Edit Catalog Service'))
         conn_edit.leName.setText(current_text)
         conn_edit.leURL.setText(url)
-        conn_edit.leUsername.setText(self.settings.value('/MetaSearch/%s/username' % current_text))
-        conn_edit.lePassword.setText(self.settings.value('/MetaSearch/%s/password' % current_text))
+        conn_edit.leUsername.setText(
+            self.settings.value('/MetaSearch/%s/username' % current_text))
+        conn_edit.lePassword.setText(
+            self.settings.value('/MetaSearch/%s/password' % current_text))
+
+        conn_edit.cmbCatalogType.setCurrentText(
+            self.settings.value('/MetaSearch/%s/catalog-type' % current_text))
 
         if conn_edit.exec_() == QDialog.Accepted:  # update service list
             self.populate_connection_list()
@@ -323,8 +338,9 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
 
         msg = self.tr('Remove service {0}?').format(current_text)
 
-        result = QMessageBox.question(self, self.tr('Delete Service'), msg,
-                                      QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
+        result = QMessageBox.question(
+            self, self.tr('Delete Service'), msg,
+            QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
         if result == QMessageBox.Yes:  # remove service from list
             self.settings.remove(key)
             index_to_delete = self.cmbConnectionsServices.currentIndex()
@@ -343,6 +359,7 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
 
         filename = os.path.join(self.context.ppath,
                                 'resources', 'connections-default.xml')
+
         doc = get_connections_from_file(self, filename)
         if doc is None:
             return
@@ -365,6 +382,7 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
             # no dups detected or overwrite is allowed
             key = '/MetaSearch/%s' % name
             self.settings.setValue('%s/url' % key, server.attrib.get('url'))
+            self.settings.setValue('%s/catalog-type' % key, server.attrib.get('catalog-type', 'OGC CSW 2.0.2'))
 
         self.populate_connection_list()
 
@@ -433,14 +451,7 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         self.constraints = []
 
         # clear all fields and disable buttons
-        self.lblResults.clear()
-        self.treeRecords.clear()
-
-        self.reset_buttons()
-
-        # save some settings
-        self.settings.setValue('/MetaSearch/returnRecords',
-                               self.spnRecords.cleanText())
+        self.clear_results()
 
         # set current catalog
         current_text = self.cmbConnectionsSearch.currentText()
@@ -448,13 +459,10 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         self.catalog_url = self.settings.value('%s/url' % key)
         self.catalog_username = self.settings.value('%s/username' % key)
         self.catalog_password = self.settings.value('%s/password' % key)
+        self.catalog_type = self.settings.value('%s/catalog-type' % key)
 
         # start position and number of records to return
-        self.startfrom = 0
-        self.maxrecords = self.spnRecords.value()
-
-        # set timeout
-        self.timeout = self.spnTimeout.value()
+        self.startfrom = 1
 
         # bbox
         # CRS is WGS84 with axis order longitude, latitude
@@ -464,43 +472,25 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         maxx = self.leEast.text()
         maxy = self.leNorth.text()
         bbox = [minx, miny, maxx, maxy]
-
-        # only apply spatial filter if bbox is not global
-        # even for a global bbox, if a spatial filter is applied, then
-        # the CSW server will skip records without a bbox
-        if bbox != ['-180', '-90', '180', '90']:
-            self.constraints.append(BBox([miny, minx, maxy, maxx],
-                                         crs='urn:ogc:def:crs:EPSG::4326'))
-
-        # keywords
-        if self.leKeywords.text():
-            # TODO: handle multiple word searches
-            keywords = self.leKeywords.text()
-            self.constraints.append(PropertyIsLike('csw:AnyText', keywords))
-
-        if len(self.constraints) > 1:  # exclusive search (a && b)
-            self.constraints = [self.constraints]
+        keywords = self.leKeywords.text()
 
         # build request
-        if not self._get_csw():
+        if not self._get_catalog():
             return
 
         # TODO: allow users to select resources types
         # to find ('service', 'dataset', etc.)
         try:
             with OverrideCursor(Qt.WaitCursor):
-                self.catalog.getrecords2(constraints=self.constraints,
-                                         maxrecords=self.maxrecords, esn='full')
-        except ExceptionReport as err:
+                self.catalog.query_records(bbox, keywords, self.maxrecords,
+                                           self.startfrom)
+
+        except Exception as err:
             QMessageBox.warning(self, self.tr('Search error'),
                                 self.tr('Search error: {0}').format(err))
             return
-        except Exception as err:
-            QMessageBox.warning(self, self.tr('Connection error'),
-                                self.tr('Connection error: {0}').format(err))
-            return
 
-        if self.catalog.results['matches'] == 0:
+        if self.catalog.matches == 0:
             self.lblResults.setText(self.tr('0 results'))
             return
 
@@ -511,30 +501,27 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
 
         self.treeRecords.clear()
 
-        position = self.catalog.results['returned'] + self.startfrom
+        position = self.catalog.returned + self.startfrom - 1
 
         msg = self.tr('Showing {0} - {1} of %n result(s)', 'number of results',
-                      self.catalog.results['matches']).format(self.startfrom + 1,
-                                                              position)
+                      self.catalog.matches).format(self.startfrom, position)
 
         self.lblResults.setText(msg)
 
-        for rec in self.catalog.records:
+        for rec in self.catalog.records():
             item = QTreeWidgetItem(self.treeRecords)
-            if self.catalog.records[rec].type:
-                item.setText(0, normalize_text(self.catalog.records[rec].type))
+            if rec['type']:
+                item.setText(0, normalize_text(rec['type']))
             else:
                 item.setText(0, 'unknown')
-            if self.catalog.records[rec].title:
-                item.setText(1,
-                             normalize_text(self.catalog.records[rec].title))
-            if self.catalog.records[rec].identifier:
-                set_item_data(item, 'identifier',
-                              self.catalog.records[rec].identifier)
+            if rec['title']:
+                item.setText(1, normalize_text(rec['title']))
+            if rec['identifier']:
+                set_item_data(item, 'identifier', rec['identifier'])
 
-        self.btnShowXml.setEnabled(True)
+        self.btnViewRawAPIResponse.setEnabled(True)
 
-        if self.catalog.results["matches"] < self.maxrecords:
+        if self.catalog.matches < self.maxrecords:
             disabled = False
         else:
             disabled = True
@@ -543,11 +530,14 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         self.btnPrev.setEnabled(disabled)
         self.btnNext.setEnabled(disabled)
         self.btnLast.setEnabled(disabled)
+        self.btnRawAPIResponse.setEnabled(False)
 
     def clear_results(self):
         """clear search results"""
 
+        self.lblResults.clear()
         self.treeRecords.clear()
+        self.reset_buttons()
 
     def record_clicked(self):
         """record clicked signal"""
@@ -555,6 +545,8 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         # disable only service buttons
         self.reset_buttons(True, False, False)
 
+        self.rubber_band.reset()
+
         if not self.treeRecords.selectedItems():
             return
 
@@ -564,22 +556,27 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
 
         identifier = get_item_data(item, 'identifier')
         try:
-            record = self.catalog.records[identifier]
-        except KeyError as err:
+            record = next(item for item in self.catalog.records()
+                          if item['identifier'] == identifier)
+        except KeyError:
             QMessageBox.warning(self,
                                 self.tr('Record parsing error'),
                                 'Unable to locate record identifier')
             return
 
         # if the record has a bbox, show a footprint on the map
-        if record.bbox is not None:
-            points = bbox_to_polygon(record.bbox)
-            if points is not None:
+        if record['bbox'] is not None:
+            bx = record['bbox']
+            rt = QgsRectangle(float(bx['minx']), float(bx['miny']),
+                              float(bx['maxx']), float(bx['maxy']))
+            geom = QgsGeometry.fromRect(rt)
+
+            if geom is not None:
                 src = QgsCoordinateReferenceSystem("EPSG:4326")
                 dst = self.map.mapSettings().destinationCrs()
-                geom = QgsGeometry.fromWkt(points)
                 if src.postgisSrid() != dst.postgisSrid():
-                    ctr = QgsCoordinateTransform(src, dst, QgsProject.instance())
+                    ctr = QgsCoordinateTransform(
+                        src, dst, QgsProject.instance())
                     try:
                         geom.transform(ctr)
                     except Exception as err:
@@ -595,11 +592,9 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
     def find_services(self, record, item):
         """scan record for WMS/WMTS|WFS|WCS endpoints"""
 
-        links = record.uris + record.references
-
         services = {}
-        for link in links:
-
+        for link in record['links']:
+            link = self.catalog.parse_link(link)
             if 'scheme' in link:
                 link_type = link['scheme']
             elif 'protocol' in link:
@@ -610,18 +605,22 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
             if link_type is not None:
                 link_type = link_type.upper()
 
-            wmswmst_link_types = list(map(str.upper, link_types.WMSWMST_LINK_TYPES))
+            wmswmst_link_types = list(
+                map(str.upper, link_types.WMSWMST_LINK_TYPES))
             wfs_link_types = list(map(str.upper, link_types.WFS_LINK_TYPES))
             wcs_link_types = list(map(str.upper, link_types.WCS_LINK_TYPES))
             ams_link_types = list(map(str.upper, link_types.AMS_LINK_TYPES))
             afs_link_types = list(map(str.upper, link_types.AFS_LINK_TYPES))
-            gis_file_link_types = list(map(str.upper, link_types.GIS_FILE_LINK_TYPES))
+            gis_file_link_types = list(
+                map(str.upper, link_types.GIS_FILE_LINK_TYPES))
 
             # if the link type exists, and it is one of the acceptable
             # interactive link types, then set
-            if all([link_type is not None,
-                    link_type in wmswmst_link_types + wfs_link_types +
-                    wcs_link_types + ams_link_types + afs_link_types + gis_file_link_types]):
+            all_link_types = (wmswmst_link_types + wfs_link_types +
+                              wcs_link_types + ams_link_types +
+                              afs_link_types + gis_file_link_types)
+
+            if all([link_type is not None, link_type in all_link_types]):
                 if link_type in wmswmst_link_types:
                     services['wms'] = link['url']
                     self.mActionAddWms.setEnabled(True)
@@ -639,7 +638,7 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
                     self.mActionAddAfs.setEnabled(True)
                 if link_type in gis_file_link_types:
                     services['gis_file'] = link['url']
-                    services['title'] = record.title
+                    services['title'] = record.get('title', '')
                     self.mActionAddGisFile.setEnabled(True)
                 self.tbAddData.setEnabled(True)
 
@@ -651,48 +650,58 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         caller = self.sender().objectName()
 
         if caller == 'btnFirst':
-            self.startfrom = 0
+            self.startfrom = 1
         elif caller == 'btnLast':
-            self.startfrom = self.catalog.results['matches'] - self.maxrecords
+            self.startfrom = self.catalog.matches - self.maxrecords + 1
         elif caller == 'btnNext':
-            self.startfrom += self.maxrecords
-            if self.startfrom >= self.catalog.results["matches"]:
+            if self.startfrom > self.catalog.matches - self.maxrecords:
                 msg = self.tr('End of results. Go to start?')
                 res = QMessageBox.information(self, self.tr('Navigation'),
                                               msg,
                                               (QMessageBox.Ok |
                                                QMessageBox.Cancel))
                 if res == QMessageBox.Ok:
-                    self.startfrom = 0
+                    self.startfrom = 1
                 else:
                     return
+            else:
+                self.startfrom += self.maxrecords
         elif caller == "btnPrev":
-            self.startfrom -= self.maxrecords
-            if self.startfrom <= 0:
+            if self.startfrom == 1:
                 msg = self.tr('Start of results. Go to end?')
                 res = QMessageBox.information(self, self.tr('Navigation'),
                                               msg,
                                               (QMessageBox.Ok |
                                                QMessageBox.Cancel))
                 if res == QMessageBox.Ok:
-                    self.startfrom = (self.catalog.results['matches'] -
-                                      self.maxrecords)
+                    self.startfrom = (self.catalog.matches -
+                                      self.maxrecords + 1)
                 else:
                     return
+            elif self.startfrom <= self.maxrecords:
+                self.startfrom = 1
+            else:
+                self.startfrom -= self.maxrecords
+
+        # bbox
+        # CRS is WGS84 with axis order longitude, latitude
+        # defined by 'urn:ogc:def:crs:OGC:1.3:CRS84'
+        minx = self.leWest.text()
+        miny = self.leSouth.text()
+        maxx = self.leEast.text()
+        maxy = self.leNorth.text()
+        bbox = [minx, miny, maxx, maxy]
+        keywords = self.leKeywords.text()
 
         try:
             with OverrideCursor(Qt.WaitCursor):
-                self.catalog.getrecords2(constraints=self.constraints,
-                                         maxrecords=self.maxrecords,
-                                         startposition=self.startfrom, esn='full')
-        except ExceptionReport as err:
+                self.catalog.query_records(bbox, keywords,
+                                           limit=self.maxrecords,
+                                           offset=self.startfrom)
+        except Exception as err:
             QMessageBox.warning(self, self.tr('Search error'),
                                 self.tr('Search error: {0}').format(err))
             return
-        except Exception as err:
-            QMessageBox.warning(self, self.tr('Connection error'),
-                                self.tr('Connection error: {0}').format(err))
-            return
 
         self.display_results()
 
@@ -725,7 +734,8 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
             data_url = item_data['ams'].split('MapServer')[0] + 'MapServer'
         elif caller == 'mActionAddAfs':
             stype = ['ESRI:ArcGIS:FeatureServer', 'afs', 'arcgisfeatureserver']
-            data_url = item_data['afs'].split('FeatureServer')[0] + 'FeatureServer'
+            data_url = (item_data['afs'].split('FeatureServer')[0] +
+                        'FeatureServer')
 
         sname = '%s from MetaSearch' % stype[1]
 
@@ -747,8 +757,9 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         # check for duplicates
         if sname in keys:  # duplicate found
             msg = self.tr('Connection {0} exists. Overwrite?').format(sname)
-            res = QMessageBox.warning(self, self.tr('Saving server'), msg,
-                                      QMessageBox.Yes | QMessageBox.No | QMessageBox.Cancel)
+            res = QMessageBox.warning(
+                self, self.tr('Saving server'), msg,
+                QMessageBox.Yes | QMessageBox.No | QMessageBox.Cancel)
             if res == QMessageBox.No:  # assign new name with serial
                 sname = serialize_string(sname)
             elif res == QMessageBox.Cancel:
@@ -763,11 +774,10 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         self.settings.endGroup()
 
         # open provider window
-        ows_provider = QgsGui.sourceSelectProviderRegistry().createSelectionWidget(
-            stype[2],
-            self,
-            Qt.Widget,
-            QgsProviderRegistry.WidgetMode.Embedded)
+        ows_provider = QgsGui.sourceSelectProviderRegistry().\
+            createSelectionWidget(
+                stype[2], self, Qt.Widget,
+                QgsProviderRegistry.WidgetMode.Embedded)
         service_type = stype[0]
 
         # connect dialog signals to iface slots
@@ -777,7 +787,7 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
             connect = 'btnConnect_clicked'
         elif service_type == 'OGC:WFS':
             def addVectorLayer(path, name):
-                self.iface.mainWindow().addVectorLayer(path, name, 'WFS')
+                self.iface.addVectorLayer(path, name, 'WFS')
 
             ows_provider.addVectorLayer.connect(addVectorLayer)
             conn_cmb = ows_provider.findChild(QWidget, 'cmbConnections')
@@ -792,7 +802,7 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
             connect = 'connectToServer'
         elif service_type == 'ESRI:ArcGIS:FeatureServer':
             def addAfsLayer(path, name):
-                self.iface.mainWindow().addVectorLayer(path, name, 'afs')
+                self.iface.addVectorLayer(path, name, 'afs')
 
             ows_provider.addVectorLayer.connect(addAfsLayer)
             conn_cmb = ows_provider.findChild(QComboBox)
@@ -808,7 +818,7 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
             # only for wfs
             if service_type == 'OGC:WFS':
                 ows_provider.cmbConnections_activated(index)
-            elif service_type in ['ESRI:ArcGIS:MapServer', 'ESRI:ArcGIS:FeatureServer']:
+            elif service_type in ['ESRI:ArcGIS:MapServer', 'ESRI:ArcGIS:FeatureServer']:  # noqa
                 ows_provider.cmbConnections_activated(index)
         getattr(ows_provider, connect)()
 
@@ -840,7 +850,6 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
 
         identifier = get_item_data(item, 'identifier')
 
-        self.disable_ssl_verification = self.disableSSLVerification.isChecked()
         auth = None
 
         if self.disable_ssl_verification:
@@ -851,50 +860,56 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
 
         try:
             with OverrideCursor(Qt.WaitCursor):
-                cat = CatalogueServiceWeb(self.catalog_url, timeout=self.timeout,  # spellok
-                                          username=self.catalog_username,
-                                          password=self.catalog_password,
+                cat = get_catalog_service(self.catalog_url,  # spellok
+                                          catalog_type=self.catalog_type,
+                                          timeout=self.timeout,
+                                          username=self.catalog_username or None,
+                                          password=self.catalog_password or None,
                                           auth=auth)
-                cat.getrecordbyid(
-                    [self.catalog.records[identifier].identifier])
-        except ExceptionReport as err:
-            QMessageBox.warning(self, self.tr('GetRecords error'),
-                                self.tr('Error getting response: {0}').format(err))
+                record = cat.get_record(identifier)
+                if cat.type == 'OGC API - Records':
+                    record['url'] = cat.conn.request
+                elif cat.type == 'OGC CSW 2.0.2':
+                    record.url = cat.conn.request
+
+        except Exception as err:
+            QMessageBox.warning(
+                self, self.tr('GetRecords error'),
+                self.tr('Error getting response: {0}').format(err))
             return
         except KeyError as err:
-            QMessageBox.warning(self,
-                                self.tr('Record parsing error'),
-                                self.tr('Unable to locate record identifier'))
+            QMessageBox.warning(
+                self, self.tr('Record parsing error'),
+                self.tr('Unable to locate record identifier: {0}').format(err))
             return
 
-        record = cat.records[identifier]
-        record.xml_url = cat.request
-
         crd = RecordDialog()
         metadata = render_template('en', self.context,
-                                   record, 'record_metadata_dc.html')
+                                   record, self.catalog.record_info_template)
 
         style = QgsApplication.reportStyleSheet()
         crd.textMetadata.document().setDefaultStyleSheet(style)
         crd.textMetadata.setHtml(metadata)
         crd.exec_()
 
-    def show_xml(self):
-        """show XML request / response"""
+    def show_api(self):
+        """show API request / response"""
 
-        crd = XMLDialog()
-        request_html = highlight_xml(self.context, self.catalog.request)
-        response_html = highlight_xml(self.context, self.catalog.response)
+        crd = APIRequestResponseDialog()
+        request_html = highlight_content(self.context, self.catalog.request,
+                                         self.catalog.format)
+        response_html = highlight_content(self.context, self.catalog.response,
+                                          self.catalog.format)
         style = QgsApplication.reportStyleSheet()
-        crd.txtbrXMLRequest.clear()
-        crd.txtbrXMLResponse.clear()
-        crd.txtbrXMLRequest.document().setDefaultStyleSheet(style)
-        crd.txtbrXMLResponse.document().setDefaultStyleSheet(style)
-        crd.txtbrXMLRequest.setHtml(request_html)
-        crd.txtbrXMLResponse.setHtml(response_html)
+        crd.txtbrAPIRequest.clear()
+        crd.txtbrAPIResponse.clear()
+        crd.txtbrAPIRequest.document().setDefaultStyleSheet(style)
+        crd.txtbrAPIResponse.document().setDefaultStyleSheet(style)
+        crd.txtbrAPIRequest.setHtml(request_html)
+        crd.txtbrAPIResponse.setHtml(response_html)
         crd.exec_()
 
-    def reset_buttons(self, services=True, xml=True, navigation=True):
+    def reset_buttons(self, services=True, api=True, navigation=True):
         """Convenience function to disable WMS/WMTS|WFS|WCS buttons"""
 
         if services:
@@ -906,8 +921,8 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
             self.mActionAddAfs.setEnabled(False)
             self.mActionAddGisFile.setEnabled(False)
 
-        if xml:
-            self.btnShowXml.setEnabled(False)
+        if api:
+            self.btnViewRawAPIResponse.setEnabled(False)
 
         if navigation:
             self.btnFirst.setEnabled(False)
@@ -926,10 +941,9 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         QDialog.reject(self)
         self.rubber_band.reset()
 
-    def _get_csw(self):
-        """convenience function to init owslib.csw.CatalogueServiceWeb"""  # spellok
+    def _get_catalog(self):
+        """convenience function to init catalog wrapper"""
 
-        self.disable_ssl_verification = self.disableSSLVerification.isChecked()
         auth = None
 
         if self.disable_ssl_verification:
@@ -941,18 +955,13 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         # connect to the server
         with OverrideCursor(Qt.WaitCursor):
             try:
-                self.catalog = CatalogueServiceWeb(self.catalog_url,  # spellok
-                                                   timeout=self.timeout,
-                                                   username=self.catalog_username,
-                                                   password=self.catalog_password,
-                                                   auth=auth)
+                self.catalog = get_catalog_service(
+                    self.catalog_url, catalog_type=self.catalog_type,
+                    timeout=self.timeout, username=self.catalog_username or None,
+                    password=self.catalog_password or None, auth=auth)
                 return True
-            except ExceptionReport as err:
-                msg = self.tr('Error connecting to service: {0}').format(err)
-            except ValueError as err:
-                msg = self.tr('Value Error: {0}').format(err)
             except Exception as err:
-                msg = self.tr('Unknown Error: {0}').format(err)
+                msg = self.tr('Error connecting to service: {0}').format(err)
 
         QMessageBox.warning(self, self.tr('CSW Connection error'), msg)
         return False
@@ -1014,20 +1023,3 @@ def _get_field_value(field):
         value = 1
 
     return value
-
-
-def bbox_to_polygon(bbox):
-    """converts OWSLib bbox object to list of QgsPointXY objects"""
-
-    if all([bbox.minx is not None,
-            bbox.maxx is not None,
-            bbox.miny is not None,
-            bbox.maxy is not None]):
-        minx = float(bbox.minx)
-        miny = float(bbox.miny)
-        maxx = float(bbox.maxx)
-        maxy = float(bbox.maxy)
-
-        return 'POLYGON((%.2f %.2f, %.2f %.2f, %.2f %.2f, %.2f %.2f, %.2f %.2f))' % (minx, miny, minx, maxy, maxx, maxy, maxx, miny, minx, miny)  # noqa
-    else:
-        return None
